@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -47,11 +47,13 @@ import tempfile
 import atexit
 import datetime
 import tokenize
+import argparse
+import typing
 
 from PyQt5.QtWidgets import QApplication, QWidget
-from PyQt5.QtGui import QDesktopServices, QPixmap, QIcon, QWindow
-from PyQt5.QtCore import (pyqtSlot, qInstallMessageHandler, QTimer, QUrl,
-                          QObject, QEvent, pyqtSignal, Qt)
+from PyQt5.QtGui import QDesktopServices, QPixmap, QIcon, QWindow, QKeyEvent
+from PyQt5.QtCore import (pyqtSlot, QTimer, QUrl, QObject, QEvent, pyqtSignal,
+                          Qt)
 try:
     import hunter
 except ImportError:
@@ -69,11 +71,11 @@ from qutebrowser.browser.network import proxy
 from qutebrowser.browser.webkit import cookies, cache
 from qutebrowser.browser.webkit.network import networkmanager
 from qutebrowser.extensions import loader
-from qutebrowser.keyinput import macros
+from qutebrowser.keyinput import macros, modeman
 from qutebrowser.mainwindow import mainwindow, prompt
-from qutebrowser.misc import (readline, ipc, savemanager, sessions,
-                              crashsignal, earlyinit, sql, cmdhistory,
-                              backendproblem, objects)
+from qutebrowser.misc import (ipc, savemanager, sessions, crashsignal,
+                              earlyinit, sql, cmdhistory, backendproblem,
+                              objects)
 from qutebrowser.utils import (log, version, message, utils, urlutils, objreg,
                                usertypes, standarddir, error, qtutils)
 # pylint: disable=unused-import
@@ -83,7 +85,7 @@ from qutebrowser.misc import utilcmds
 # pylint: enable=unused-import
 
 
-q_app = None
+q_app = typing.cast(QApplication, None)
 
 
 def run(args):
@@ -91,8 +93,9 @@ def run(args):
     if args.temp_basedir:
         args.basedir = tempfile.mkdtemp(prefix='qutebrowser-basedir-')
 
-    quitter = Quitter(args)
-    objreg.register('quitter', quitter)
+    quitter = Quitter(args=args)
+    objreg.register('quitter', quitter, command_only=True)
+    quitter.shutting_down.connect(log.shutdown_log)
 
     log.init.debug("Initializing directories...")
     standarddir.init(args)
@@ -107,7 +110,8 @@ def run(args):
     q_app.setApplicationName("qutebrowser")
     q_app.setDesktopFileName("org.qutebrowser.qutebrowser")
     q_app.setApplicationVersion(qutebrowser.__version__)
-    q_app.lastWindowClosed.connect(quitter.on_last_window_closed)
+    q_app.lastWindowClosed.connect(  # type: ignore
+        quitter.on_last_window_closed)
 
     if args.version:
         print(version.version())
@@ -115,13 +119,14 @@ def run(args):
 
     crash_handler = crashsignal.CrashHandler(
         app=q_app, quitter=quitter, args=args, parent=q_app)
+    objreg.register('crash-handler', crash_handler, command_only=True)
     crash_handler.activate()
-    objreg.register('crash-handler', crash_handler)
+    quitter.shutting_down.connect(crash_handler.shutdown)
 
     signal_handler = crashsignal.SignalHandler(app=q_app, quitter=quitter,
                                                parent=q_app)
     signal_handler.activate()
-    objreg.register('signal-handler', signal_handler)
+    quitter.shutting_down.connect(signal_handler.deactivate)
 
     try:
         server = ipc.send_or_listen(args)
@@ -136,11 +141,12 @@ def run(args):
                 "Backend from the running instance will be used")
         sys.exit(usertypes.Exit.ok)
     else:
+        quitter.shutting_down.connect(server.shutdown)
         server.got_args.connect(lambda args, target_arg, cwd:
                                 process_pos_args(args, cwd=cwd, via_ipc=True,
                                                  target_arg=target_arg))
 
-    init(args, crash_handler)
+    init(args=args, crash_handler=crash_handler, quitter=quitter)
     ret = qt_mainloop()
     return ret
 
@@ -154,24 +160,23 @@ def qt_mainloop():
     return q_app.exec_()
 
 
-def init(args, crash_handler):
-    """Initialize everything.
-
-    Args:
-        args: The argparse namespace.
-        crash_handler: The CrashHandler instance.
-    """
+def init(*, args: argparse.Namespace,
+         crash_handler: crashsignal.CrashHandler,
+         quitter: 'Quitter'):
+    """Initialize everything."""
     log.init.debug("Starting init...")
 
     crash_handler.init_faulthandler()
 
     q_app.setQuitOnLastWindowClosed(False)
+    quitter.shutting_down.connect(QApplication.closeAllWindows)
+
     _init_icon()
 
     loader.init()
     loader.load_components()
     try:
-        _init_modules(args, crash_handler)
+        _init_modules(args=args, crash_handler=crash_handler, quitter=quitter)
     except (OSError, UnicodeDecodeError, browsertab.WebTabError) as e:
         error.handle_fatal_exc(e, args, "Error while initializing!",
                                pre_text="Error while initializing")
@@ -179,17 +184,17 @@ def init(args, crash_handler):
 
     log.init.debug("Initializing eventfilter...")
     event_filter = EventFilter(q_app)
-    q_app.installEventFilter(event_filter)
-    objreg.register('event-filter', event_filter)
+    event_filter.install()
+    quitter.shutting_down.connect(event_filter.shutdown)
 
     log.init.debug("Connecting signals...")
-    q_app.focusChanged.connect(on_focus_changed)
+    q_app.focusChanged.connect(on_focus_changed)  # type: ignore
 
     _process_args(args)
 
-    QDesktopServices.setUrlHandler('http', open_desktopservices_url)
-    QDesktopServices.setUrlHandler('https', open_desktopservices_url)
-    QDesktopServices.setUrlHandler('qute', open_desktopservices_url)
+    for scheme in ['http', 'https', 'qute']:
+        QDesktopServices.setUrlHandler(
+            scheme, open_desktopservices_url)  # type: ignore
 
     log.init.debug("Init done!")
     crash_handler.raise_crashdlg()
@@ -217,8 +222,8 @@ def _process_args(args):
     """Open startpage etc. and process commandline args."""
     if not args.override_restore:
         _load_session(args.session)
-    session_manager = objreg.get('session-manager')
-    if not session_manager.did_load:
+
+    if not sessions.session_manager.did_load:
         log.init.debug("Initializing main window...")
         if config.val.content.private_browsing and qtutils.is_single_process():
             err = Exception("Private windows are unavailable with "
@@ -244,8 +249,7 @@ def _load_session(name):
     Args:
         name: The name of the session to load, or None to read state file.
     """
-    session_manager = objreg.get('session-manager')
-    if name is None and session_manager.exists('_autosave'):
+    if name is None and sessions.session_manager.exists('_autosave'):
         name = '_autosave'
     elif name is None:
         try:
@@ -256,7 +260,7 @@ def _load_session(name):
             return
 
     try:
-        session_manager.load(name)
+        sessions.session_manager.load(name)
     except sessions.SessionNotFoundError:
         message.error("Session {} not found!".format(name))
     except sessions.SessionError as e:
@@ -267,7 +271,7 @@ def _load_session(name):
         pass
     # If this was a _restart session, delete it.
     if name == '_restart':
-        session_manager.delete('_restart')
+        sessions.session_manager.delete('_restart')
 
 
 def process_pos_args(args, via_ipc=False, cwd=None, target_arg=None):
@@ -347,7 +351,7 @@ def _open_startpage(win_id=None):
                 If set, open the startpage in the given window.
     """
     if win_id is not None:
-        window_ids = [win_id]
+        window_ids = [win_id]  # type: typing.Iterable[int]
     else:
         window_ids = objreg.window_registry
     for cur_win_id in list(window_ids):  # Copying as the dict could change
@@ -425,20 +429,22 @@ def open_desktopservices_url(url):
     tabbed_browser.tabopen(url)
 
 
-def _init_modules(args, crash_handler):
+def _init_modules(*, args, crash_handler, quitter):
     """Initialize all 'modules' which need to be initialized.
 
     Args:
         args: The argparse namespace.
         crash_handler: The CrashHandler instance.
+        quitter: The Quitter instance.
     """
     log.init.debug("Initializing save manager...")
     save_manager = savemanager.SaveManager(q_app)
     objreg.register('save-manager', save_manager)
+    quitter.shutting_down.connect(save_manager.shutdown)
     configinit.late_init(save_manager)
 
     log.init.debug("Checking backend requirements...")
-    backendproblem.init()
+    backendproblem.init(quitter=quitter, args=args, save_manager=save_manager)
 
     log.init.debug("Initializing prompts...")
     prompt.init()
@@ -448,13 +454,11 @@ def _init_modules(args, crash_handler):
 
     log.init.debug("Initializing proxy...")
     proxy.init()
+    quitter.shutting_down.connect(proxy.shutdown)
 
     log.init.debug("Initializing downloads...")
     downloads.init()
-
-    log.init.debug("Initializing readline-bridge...")
-    readline_bridge = readline.ReadlineBridge()
-    objreg.register('readline-bridge', readline_bridge)
+    quitter.shutting_down.connect(downloads.shutdown)
 
     try:
         log.init.debug("Initializing SQL...")
@@ -471,9 +475,11 @@ def _init_modules(args, crash_handler):
     cmdhistory.init()
     log.init.debug("Initializing sessions...")
     sessions.init(q_app)
+    quitter.shutting_down.connect(sessions.shutdown)
 
     log.init.debug("Initializing websettings...")
     websettings.init(args)
+    quitter.shutting_down.connect(websettings.shutdown)
 
     if not args.no_err_windows:
         crash_handler.display_faulthandler()
@@ -487,14 +493,10 @@ def _init_modules(args, crash_handler):
     objreg.register('bookmark-manager', bookmark_manager)
 
     log.init.debug("Initializing cookies...")
-    cookie_jar = cookies.CookieJar(q_app)
-    ram_cookie_jar = cookies.RAMCookieJar(q_app)
-    objreg.register('cookie-jar', cookie_jar)
-    objreg.register('ram-cookie-jar', ram_cookie_jar)
+    cookies.init(q_app)
 
     log.init.debug("Initializing cache...")
-    diskcache = cache.DiskCache(standarddir.cache(), parent=q_app)
-    objreg.register('cache', diskcache)
+    cache.init(q_app)
 
     log.init.debug("Initializing downloads...")
     download_manager = qtnetworkdownloads.DownloadManager(parent=q_app)
@@ -509,30 +511,35 @@ def _init_modules(args, crash_handler):
     browsertab.init()
 
 
-class Quitter:
+class Quitter(QObject):
 
     """Utility class to quit/restart the QApplication.
 
     Attributes:
         quit_status: The current quitting status.
-        _shutting_down: Whether we're currently shutting down.
+        _is_shutting_down: Whether we're currently shutting down.
         _args: The argparse namespace.
     """
 
-    def __init__(self, args):
+    shutting_down = pyqtSignal()  # Emitted immediately before shut down
+
+    def __init__(self, *,
+                 args: argparse.Namespace,
+                 parent: QObject = None) -> None:
+        super().__init__(parent)
         self.quit_status = {
             'crash': True,
             'tabs': False,
             'main': False,
         }
-        self._shutting_down = False
+        self._is_shutting_down = False
         self._args = args
 
-    def on_last_window_closed(self):
+    def on_last_window_closed(self) -> None:
         """Slot which gets invoked when the last window was closed."""
         self.shutdown(last_window=True)
 
-    def _compile_modules(self):
+    def _compile_modules(self) -> None:
         """Compile all modules to catch SyntaxErrors."""
         if os.path.basename(sys.argv[0]) == 'qutebrowser':
             # Launched via launcher script
@@ -551,8 +558,12 @@ class Quitter:
                     with tokenize.open(os.path.join(dirpath, fn)) as f:
                         compile(f.read(), fn, 'exec')
 
-    def _get_restart_args(self, pages=(), session=None, override_args=None):
-        """Get the current working directory and args to relaunch qutebrowser.
+    def _get_restart_args(
+            self, pages: typing.Iterable[str] = (),
+            session: str = None,
+            override_args: typing.Mapping[str, str] = None
+    ) -> typing.Sequence[str]:
+        """Get args to relaunch qutebrowser.
 
         Args:
             pages: The pages to re-open.
@@ -560,29 +571,18 @@ class Quitter:
             override_args: Argument overrides as a dict.
 
         Return:
-            An (args, cwd) tuple.
-                args: The commandline as a list of strings.
-                cwd: The current working directory as a string.
+            The commandline as a list of strings.
         """
         if os.path.basename(sys.argv[0]) == 'qutebrowser':
             # Launched via launcher script
             args = [sys.argv[0]]
-            cwd = None
         elif hasattr(sys, 'frozen'):
             args = [sys.executable]
-            cwd = os.path.abspath(os.path.dirname(sys.executable))
         else:
             args = [sys.executable, '-m', 'qutebrowser']
-            cwd = os.path.join(
-                os.path.abspath(os.path.dirname(qutebrowser.__file__)), '..')
-            if not os.path.isdir(cwd):
-                # Probably running from a python egg. Let's fallback to
-                # cwd=None and see if that works out.
-                # See https://github.com/qutebrowser/qutebrowser/issues/323
-                cwd = None
 
         # Add all open pages so they get reopened.
-        page_args = []
+        page_args = []  # type: typing.MutableSequence[str]
         for win in pages:
             page_args.extend(win)
             page_args.append('')
@@ -616,12 +616,11 @@ class Quitter:
         args += ['--json-args', data]
 
         log.destroy.debug("args: {}".format(args))
-        log.destroy.debug("cwd: {}".format(cwd))
 
-        return args, cwd
+        return args
 
     @cmdutils.register(instance='quitter', name='restart')
-    def restart_cmd(self):
+    def restart_cmd(self) -> None:
         """Restart qutebrowser while keeping existing tabs open."""
         try:
             ok = self.restart(session='_restart')
@@ -636,7 +635,9 @@ class Quitter:
         if ok:
             self.shutdown(restart=True)
 
-    def restart(self, pages=(), session=None, override_args=None):
+    def restart(self, pages: typing.Sequence[str] = (),
+                session: str = None,
+                override_args: typing.Mapping[str, str] = None) -> bool:
         """Inner logic to restart qutebrowser.
 
         The "better" way to restart is to pass a session (_restart usually) as
@@ -662,20 +663,17 @@ class Quitter:
 
         # Save the session if one is given.
         if session is not None:
-            session_manager = objreg.get('session-manager')
-            session_manager.save(session, with_private=True)
+            sessions.session_manager.save(session, with_private=True)
 
         # Make sure we're not accepting a connection from the new process
         # before we fully exited.
+        assert ipc.server is not None
         ipc.server.shutdown()
 
         # Open a new process and immediately shutdown the existing one
         try:
-            args, cwd = self._get_restart_args(pages, session, override_args)
-            if cwd is None:
-                subprocess.Popen(args)
-            else:
-                subprocess.Popen(args, cwd=cwd)
+            args = self._get_restart_args(pages, session, override_args)
+            subprocess.Popen(args)
         except OSError:
             log.destroy.exception("Failed to restart")
             return False
@@ -684,7 +682,9 @@ class Quitter:
 
     @cmdutils.register(instance='quitter', name='quit')
     @cmdutils.argument('session', completion=miscmodels.session)
-    def quit(self, save=False, session=None):
+    def quit(self,
+             save: bool = False,
+             session: sessions.ArgType = None) -> None:
         """Quit qutebrowser.
 
         Args:
@@ -701,8 +701,10 @@ class Quitter:
         else:
             self.shutdown()
 
-    def shutdown(self, status=0, session=None, last_window=False,
-                 restart=False):
+    def shutdown(self, status: int = 0,
+                 session: sessions.ArgType = None,
+                 last_window: bool = False,
+                 restart: bool = False) -> None:
         """Quit qutebrowser.
 
         Args:
@@ -712,19 +714,20 @@ class Quitter:
                             closing.
             restart: If we're planning to restart.
         """
-        if self._shutting_down:
+        if self._is_shutting_down:
             return
-        self._shutting_down = True
+        self._is_shutting_down = True
         log.destroy.debug("Shutting down with status {}, session {}...".format(
             status, session))
-        session_manager = objreg.get('session-manager', None)
-        if session_manager is not None:
+        if sessions.session_manager is not None:
             if session is not None:
-                session_manager.save(session, last_window=last_window,
-                                     load_next_time=True)
+                sessions.session_manager.save(session,
+                                              last_window=last_window,
+                                              load_next_time=True)
             elif config.val.auto_save.session:
-                session_manager.save(sessions.default, last_window=last_window,
-                                     load_next_time=True)
+                sessions.session_manager.save(sessions.default,
+                                              last_window=last_window,
+                                              load_next_time=True)
 
         if prompt.prompt_queue.shutdown():
             # If shutdown was called while we were asking a question, we're in
@@ -741,64 +744,24 @@ class Quitter:
             # event loop, so we can shut down immediately.
             self._shutdown(status, restart=restart)
 
-    def _shutdown(self, status, restart):  # noqa
+    def _shutdown(self, status: int, restart: bool) -> None:  # noqa
         """Second stage of shutdown."""
         log.destroy.debug("Stage 2 of shutting down...")
         if q_app is None:
             # No QApplication exists yet, so quit hard.
             sys.exit(status)
-        # Remove eventfilter
-        try:
-            log.destroy.debug("Removing eventfilter...")
-            event_filter = objreg.get('event-filter', None)
-            if event_filter is not None:
-                q_app.removeEventFilter(event_filter)
-        except AttributeError:
-            pass
-        # Close all windows
-        QApplication.closeAllWindows()
-        # Shut down IPC
-        try:
-            ipc.server.shutdown()
-        except KeyError:
-            pass
-        # Save everything
-        try:
-            save_manager = objreg.get('save-manager')
-        except KeyError:
-            log.destroy.debug("Save manager not initialized yet, so not "
-                              "saving anything.")
-        else:
-            for key in save_manager.saveables:
-                try:
-                    save_manager.save(key, is_exit=True)
-                except OSError as e:
-                    error.handle_fatal_exc(
-                        e, self._args, "Error while saving!",
-                        pre_text="Error while saving {}".format(key))
-        # Disable storage so removing tempdir will work
-        websettings.shutdown()
-        # Disable application proxy factory to fix segfaults with Qt 5.10.1
-        proxy.shutdown()
-        # Re-enable faulthandler to stdout, then remove crash log
-        log.destroy.debug("Deactivating crash log...")
-        objreg.get('crash-handler').destroy_crashlogfile()
+
+        # Tell everything to shut itself down
+        self.shutting_down.emit()
+
         # Delete temp basedir
         if ((self._args.temp_basedir or self._args.temp_basedir_restarted) and
                 not restart):
             atexit.register(shutil.rmtree, self._args.basedir,
                             ignore_errors=True)
-        # Delete temp download dir
-        downloads.temp_download_manager.cleanup()
-        # If we don't kill our custom handler here we might get segfaults
-        log.destroy.debug("Deactivating message handler...")
-        qInstallMessageHandler(None)
+
         # Now we can hopefully quit without segfaults
         log.destroy.debug("Deferring QApplication::exit...")
-        objreg.get('signal-handler').deactivate()
-        session_manager = objreg.get('session-manager', None)
-        if session_manager is not None:
-            session_manager.delete_autosave()
         # We use a singleshot timer to exit here to minimize the likelihood of
         # segfaults.
         QTimer.singleShot(0, functools.partial(q_app.exit, status))
@@ -827,13 +790,13 @@ class Application(QApplication):
         log.init.debug("Qt arguments: {}, based on {}".format(qt_args, args))
         super().__init__(qt_args)
 
+        objects.args = args
+
         log.init.debug("Initializing application...")
 
-        objreg.register('args', args)
-        objreg.register('app', self)
-
         self.launch_time = datetime.datetime.now()
-        self.focusObjectChanged.connect(self.on_focus_object_changed)
+        self.focusObjectChanged.connect(  # type: ignore
+            self.on_focus_object_changed)
         self.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
 
     @pyqtSlot(QObject)
@@ -883,7 +846,7 @@ class EventFilter(QObject):
                    event.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: QObject = None) -> None:
         super().__init__(parent)
         self._activated = True
         self._handlers = {
@@ -892,7 +855,14 @@ class EventFilter(QObject):
             QEvent.ShortcutOverride: self._handle_key_event,
         }
 
-    def _handle_key_event(self, event):
+    def install(self):
+        q_app.installEventFilter(self)
+
+    @pyqtSlot()
+    def shutdown(self):
+        q_app.removeEventFilter(self)
+
+    def _handle_key_event(self, event: QKeyEvent) -> bool:
         """Handle a key press/release event.
 
         Args:
@@ -906,13 +876,13 @@ class EventFilter(QObject):
             # event through.
             return False
         try:
-            man = objreg.get('mode-manager', scope='window', window='current')
+            man = modeman.instance('current')
             return man.handle_event(event)
         except objreg.RegistryUnavailableError:
             # No window available yet, or not a MainWindow
             return False
 
-    def eventFilter(self, obj, event):
+    def eventFilter(self, obj: QObject, event: QEvent) -> bool:
         """Handle an event.
 
         Args:
