@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -23,15 +23,17 @@ import math
 import functools
 import re
 import html as html_utils
+import typing
 
 from PyQt5.QtCore import (pyqtSignal, pyqtSlot, Qt, QPoint, QPointF, QUrl,
                           QTimer, QObject)
 from PyQt5.QtNetwork import QAuthenticator
-from PyQt5.QtWidgets import QApplication
+from PyQt5.QtWidgets import QApplication, QWidget
 from PyQt5.QtWebEngineWidgets import QWebEnginePage, QWebEngineScript
 
 from qutebrowser.config import configdata, config
-from qutebrowser.browser import browsertab, eventfilter, shared, webelem
+from qutebrowser.browser import (browsertab, eventfilter, shared, webelem,
+                                 history, greasemonkey)
 from qutebrowser.browser.webengine import (webview, webengineelem, tabhistory,
                                            interceptor, webenginequtescheme,
                                            cookies, webenginedownloads,
@@ -78,14 +80,13 @@ def init():
         cookies.install_filter(webenginesettings.private_profile)
 
     # Clear visited links on web history clear
-    hist = objreg.get('web-history')
     for p in [webenginesettings.default_profile,
               webenginesettings.private_profile]:
         if not p:
             continue
-        hist.history_cleared.connect(p.clearAllVisitedLinks)
-        hist.url_cleared.connect(lambda url, profile=p:
-                                 profile.clearVisitedLinks([url]))
+        history.web_history.history_cleared.connect(p.clearAllVisitedLinks)
+        history.web_history.url_cleared.connect(
+            lambda url, profile=p: profile.clearVisitedLinks([url]))
 
 
 # Mapping worlds from usertypes.JsWorld to QWebEngineScript world IDs.
@@ -122,7 +123,8 @@ class WebEngineAction(browsertab.AbstractAction):
             # Qt < 5.8
             tb = objreg.get('tabbed-browser', scope='window',
                             window=self._tab.win_id)
-            urlstr = self._tab.url().toString(QUrl.RemoveUserInfo)
+            urlstr = self._tab.url().toString(
+                QUrl.RemoveUserInfo)  # type: ignore
             # The original URL becomes the path of a view-source: URL
             # (without a host), but query/fragment should stay.
             url = QUrl('view-source:' + urlstr)
@@ -166,7 +168,7 @@ class WebEngineSearch(browsertab.AbstractSearch):
 
     def __init__(self, tab, parent=None):
         super().__init__(tab, parent)
-        self._flags = QWebEnginePage.FindFlags(0)
+        self._flags = QWebEnginePage.FindFlags(0)  # type: ignore
         self._pending_searches = 0
 
     def _find(self, text, flags, callback, caller):
@@ -216,7 +218,7 @@ class WebEngineSearch(browsertab.AbstractSearch):
             return
 
         self.text = text
-        self._flags = QWebEnginePage.FindFlags(0)
+        self._flags = QWebEnginePage.FindFlags(0)  # type: ignore
         if self._is_case_sensitive(ignore_case):
             self._flags |= QWebEnginePage.FindCaseSensitively
         if reverse:
@@ -232,7 +234,7 @@ class WebEngineSearch(browsertab.AbstractSearch):
 
     def prev_result(self, *, result_cb=None):
         # The int() here makes sure we get a copy of the flags.
-        flags = QWebEnginePage.FindFlags(int(self._flags))
+        flags = QWebEnginePage.FindFlags(int(self._flags))  # type: ignore
         if flags & QWebEnginePage.FindBackward:
             flags &= ~QWebEnginePage.FindBackward
         else:
@@ -757,7 +759,7 @@ class _WebEnginePermissions(QObject):
     def __init__(self, tab, parent=None):
         super().__init__(parent)
         self._tab = tab
-        self._widget = None
+        self._widget = typing.cast(QWidget, None)
 
         try:
             self._options.update({
@@ -896,8 +898,8 @@ class _WebEngineScripts(QObject):
     def __init__(self, tab, parent=None):
         super().__init__(parent)
         self._tab = tab
-        self._widget = None
-        self._greasemonkey = objreg.get('greasemonkey')
+        self._widget = typing.cast(QWidget, None)
+        self._greasemonkey = greasemonkey.gm_manager
 
     def connect_signals(self):
         """Connect signals to our private slots."""
@@ -1149,7 +1151,7 @@ class WebEngineTab(browsertab.AbstractTab):
         self.backend = usertypes.Backend.QtWebEngine
         self._child_event_filter = None
         self._saved_zoom = None
-        self._reload_url = None
+        self._reload_url = None  # type: typing.Optional[QUrl]
         self._scripts.init()
 
     def _set_widget(self, widget):
@@ -1207,8 +1209,9 @@ class WebEngineTab(browsertab.AbstractTab):
             self._widget.page().toHtml(callback)
 
     def run_js_async(self, code, callback=None, *, world=None):
+        world_id_type = typing.Union[QWebEngineScript.ScriptWorldId, int]
         if world is None:
-            world_id = QWebEngineScript.ApplicationWorld
+            world_id = QWebEngineScript.ApplicationWorld  # type: world_id_type
         elif isinstance(world, int):
             world_id = world
             if not 0 <= world_id <= qtutils.MAX_WORLD_ID:
@@ -1275,7 +1278,9 @@ class WebEngineTab(browsertab.AbstractTab):
         title = self.title()
         title_url = QUrl(url)
         title_url.setScheme('')
-        if title == title_url.toDisplayString(QUrl.RemoveScheme).strip('/'):
+        title_url_str = title_url.toDisplayString(
+            QUrl.RemoveScheme)  # type: ignore
+        if title == title_url_str.strip('/'):
             title = ""
 
         # Don't add history entry if the URL is invalid anyways
@@ -1301,7 +1306,7 @@ class WebEngineTab(browsertab.AbstractTab):
             authenticator.setPassword(answer.password)
         else:
             try:
-                sip.assign(authenticator, QAuthenticator())
+                sip.assign(authenticator, QAuthenticator())  # type: ignore
             except AttributeError:
                 self._show_error_page(url, "Proxy authentication required")
 
@@ -1322,7 +1327,7 @@ class WebEngineTab(browsertab.AbstractTab):
         if not netrc_success and answer is None:
             log.network.debug("Aborting auth")
             try:
-                sip.assign(authenticator, QAuthenticator())
+                sip.assign(authenticator, QAuthenticator())  # type: ignore
             except AttributeError:
                 # WORKAROUND for
                 # https://www.riverbankcomputing.com/pipermail/pyqt/2016-December/038400.html
@@ -1439,14 +1444,32 @@ class WebEngineTab(browsertab.AbstractTab):
             error.ignore, url, self.url(requested=True)))
 
         # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-56207
+        show_cert_error = (
+            not qtutils.version_check('5.9') and
+            not error.ignore
+        )
+        # WORKAROUND for https://codereview.qt-project.org/c/qt/qtwebengine/+/270556
+        show_non_overr_cert_error = (
+            not error.is_overridable() and (
+                # Affected Qt versions:
+                # 5.13 before 5.13.2
+                # 5.12 before 5.12.6
+                # < 5.12
+                (qtutils.version_check('5.13') and
+                 not qtutils.version_check('5.13.2')) or
+                (qtutils.version_check('5.12') and
+                 not qtutils.version_check('5.12.6')) or
+                not qtutils.version_check('5.12')
+            )
+        )
+
         # We can't really know when to show an error page, as the error might
         # have happened when loading some resource.
         # However, self.url() is not available yet and the requested URL
         # might not match the URL we get from the error - so we just apply a
         # heuristic here.
-        if (not qtutils.version_check('5.9') and
-                not error.ignore and
-                url.matches(self.url(requested=True), QUrl.RemoveScheme)):
+        if ((show_cert_error or show_non_overr_cert_error) and
+                url.matches(self.data.last_navigation.url, QUrl.RemoveScheme)):
             self._show_error_page(url, str(error))
 
     @pyqtSlot(QUrl)
@@ -1488,7 +1511,7 @@ class WebEngineTab(browsertab.AbstractTab):
         assert settings_needing_reload.issubset(configdata.DATA)
 
         changed = self.settings.update_for_url(navigation.url)
-        reload_needed = changed & settings_needing_reload
+        reload_needed = bool(changed & settings_needing_reload)
 
         # On Qt < 5.11, we don't don't need a reload when type == link_clicked.
         # On Qt 5.11.0, we always need a reload.
@@ -1555,7 +1578,7 @@ class WebEngineTab(browsertab.AbstractTab):
 
         try:
             # pylint: disable=unused-import
-            from PyQt5.QtWebEngineWidgets import (
+            from PyQt5.QtWebEngineWidgets import (  # type: ignore
                 QWebEngineClientCertificateSelection)
         except ImportError:
             pass
@@ -1574,8 +1597,8 @@ class WebEngineTab(browsertab.AbstractTab):
         page.loadFinished.connect(self._on_load_finished)
 
         self.before_load_started.connect(self._on_before_load_started)
-        self.shutting_down.connect(self.abort_questions)
-        self.load_started.connect(self.abort_questions)
+        self.shutting_down.connect(self.abort_questions)  # type: ignore
+        self.load_started.connect(self.abort_questions)  # type: ignore
 
         # pylint: disable=protected-access
         self.audio._connect_signals()
