@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -61,7 +61,7 @@ from PyQt5.QtWidgets import QTabWidget, QTabBar
 from PyQt5.QtNetwork import QNetworkProxy
 
 from qutebrowser.misc import objects, debugcachestats
-from qutebrowser.config import configexc, configutils
+from qutebrowser.config import configexc
 from qutebrowser.utils import (standarddir, utils, qtutils, urlutils, urlmatch,
                                usertypes)
 from qutebrowser.keyinput import keyutils
@@ -80,8 +80,8 @@ BOOLEAN_STATES = {'1': True, 'yes': True, 'true': True, 'on': True,
 
 
 _Completions = typing.Optional[typing.Iterable[typing.Tuple[str, str]]]
-_StrUnset = typing.Union[str, configutils.Unset]
-_StrUnsetNone = typing.Union[None, str, configutils.Unset]
+_StrUnset = typing.Union[str, usertypes.Unset]
+_StrUnsetNone = typing.Union[None, str, usertypes.Unset]
 
 
 class ValidValues:
@@ -168,7 +168,7 @@ class BaseType:
             value: The value to check.
             pytype: A Python type to check the value against.
         """
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return
 
         if (value is None or (pytype == list and value == []) or
@@ -342,7 +342,7 @@ class MappingType(BaseType):
 
     def to_py(self, value: typing.Any) -> typing.Any:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -408,7 +408,7 @@ class String(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -450,7 +450,7 @@ class UniqueCharString(String):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         py_value = super().to_py(value)
-        if isinstance(py_value, configutils.Unset):
+        if isinstance(py_value, usertypes.Unset):
             return py_value
         elif not py_value:
             return None
@@ -510,10 +510,10 @@ class List(BaseType):
 
     def to_py(
             self,
-            value: typing.Union[typing.List, configutils.Unset]
-    ) -> typing.Union[typing.List, configutils.Unset]:
+            value: typing.Union[typing.List, usertypes.Unset]
+    ) -> typing.Union[typing.List, usertypes.Unset]:
         self._basic_py_validation(value, list)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return []
@@ -601,7 +601,7 @@ class ListOrValue(BaseType):
         return value
 
     def to_py(self, value: typing.Any) -> typing.Any:
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
 
         try:
@@ -652,10 +652,10 @@ class FlagList(List):
 
     def to_py(
             self,
-            value: typing.Union[configutils.Unset, typing.List],
-    ) -> typing.Union[configutils.Unset, typing.List]:
+            value: typing.Union[usertypes.Unset, typing.List],
+    ) -> typing.Union[usertypes.Unset, typing.List]:
         vals = super().to_py(value)
-        if not isinstance(vals, configutils.Unset):
+        if not isinstance(vals, usertypes.Unset):
             self._check_duplicates(vals)
         return vals
 
@@ -765,10 +765,12 @@ class _Numeric(BaseType):  # pylint: disable=abstract-method
 
     def __init__(self, minval: int = None,
                  maxval: int = None,
+                 zero_ok: bool = True,
                  none_ok: bool = False) -> None:
         super().__init__(none_ok)
         self.minval = self._parse_bound(minval)
         self.maxval = self._parse_bound(maxval)
+        self.zero_ok = zero_ok
         if self.maxval is not None and self.minval is not None:
             if self.maxval < self.minval:
                 raise ValueError("minval ({}) needs to be <= maxval ({})!"
@@ -798,6 +800,8 @@ class _Numeric(BaseType):  # pylint: disable=abstract-method
         elif self.maxval is not None and value > self.maxval:
             raise configexc.ValidationError(
                 value, "must be {}{} or smaller!".format(self.maxval, suffix))
+        elif not self.zero_ok and value == 0:
+            raise configexc.ValidationError(value, "must not be 0!")
 
     def to_str(self, value: typing.Union[None, int, float]) -> str:
         if value is None:
@@ -862,10 +866,10 @@ class Perc(_Numeric):
 
     def to_py(
             self,
-            value: typing.Union[None, float, int, str, configutils.Unset]
-    ) -> typing.Union[None, float, int, configutils.Unset]:
+            value: typing.Union[None, float, int, str, usertypes.Unset]
+    ) -> typing.Union[None, float, int, usertypes.Unset]:
         self._basic_py_validation(value, (float, int, str))
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1058,10 +1062,10 @@ class QtColor(BaseType):
         except ValueError:
             raise configexc.ValidationError(val, "must be a valid color value")
 
-    def to_py(self, value: _StrUnset) -> typing.Union[configutils.Unset,
+    def to_py(self, value: _StrUnset) -> typing.Union[usertypes.Unset,
                                                       None, QColor]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1082,7 +1086,7 @@ class QtColor(BaseType):
             if not conv:
                 raise configexc.ValidationError(
                     value,
-                    '{} not in {}'.format(kind, list(sorted(converters))))
+                    '{} not in {}'.format(kind, sorted(converters)))
 
             if len(kind) != len(vals):
                 raise configexc.ValidationError(
@@ -1119,7 +1123,7 @@ class QssColor(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1166,7 +1170,7 @@ class Font(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1187,7 +1191,7 @@ class FontFamily(Font):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1211,23 +1215,30 @@ class QtFont(Font):
 
     __doc__ = Font.__doc__  # for src2asciidoc.py
 
-    def to_py(self, value: _StrUnset) -> typing.Union[configutils.Unset,
+    def _parse_families(self, family_str: str) -> typing.List[str]:
+        if family_str == 'monospace' and self.monospace_fonts is not None:
+            family_str = self.monospace_fonts
+
+        families = []
+        for part in family_str.split(','):
+            part = part.strip()
+            # The Qt CSS parser handles " and ' before passing the string to
+            # QFont.setFamily.
+            if ((part.startswith("'") and part.endswith("'")) or
+                    (part.startswith('"') and part.endswith('"'))):
+                part = part[1:-1]
+            families.append(part)
+
+        return families
+
+    def to_py(self, value: _StrUnset) -> typing.Union[usertypes.Unset,
                                                       None, QFont]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
 
-        style_map = {
-            'normal': QFont.StyleNormal,
-            'italic': QFont.StyleItalic,
-            'oblique': QFont.StyleOblique,
-        }
-        weight_map = {
-            'normal': QFont.Normal,
-            'bold': QFont.Bold,
-        }
         font = QFont()
         font.setStyle(QFont.StyleNormal)
         font.setWeight(QFont.Normal)
@@ -1243,13 +1254,25 @@ class QtFont(Font):
         namedweight = match.group('namedweight')
         size = match.group('size')
         family = match.group('family')
+
+        style_map = {
+            'normal': QFont.StyleNormal,
+            'italic': QFont.StyleItalic,
+            'oblique': QFont.StyleOblique,
+        }
         if style:
             font.setStyle(style_map[style])
+
+        weight_map = {
+            'normal': QFont.Normal,
+            'bold': QFont.Bold,
+        }
         if namedweight:
             font.setWeight(weight_map[namedweight])
-        if weight:
+        elif weight:
             # based on qcssparser.cpp:setFontWeightFromValue
             font.setWeight(min(int(weight) // 8, 99))
+
         if size:
             if size.lower().endswith('pt'):
                 font.setPointSizeF(float(size[:-2]))
@@ -1261,14 +1284,13 @@ class QtFont(Font):
                 raise ValueError("Unexpected size unit in {!r}!".format(
                     size))  # pragma: no cover
 
-        if family == 'monospace' and self.monospace_fonts is not None:
-            family = self.monospace_fonts
-        # The Qt CSS parser handles " and ' before passing the string to
-        # QFont.setFamily. We could do proper CSS-like parsing here, but since
-        # hopefully nobody will ever have a font with quotes in the family (if
-        # that's even possible), we take a much more naive approach.
-        family = family.replace('"', '').replace("'", '')
-        font.setFamily(family)
+        families = self._parse_families(family)
+        if hasattr(font, 'setFamilies'):
+            # Added in Qt 5.13
+            font.setFamily(families[0])
+            font.setFamilies(families)
+        else:  # pragma: no cover
+            font.setFamily(', '.join(families))
 
         return font
 
@@ -1329,11 +1351,11 @@ class Regex(BaseType):
 
     def to_py(
             self,
-            value: typing.Union[str, typing.Pattern[str], configutils.Unset]
-    ) -> typing.Union[configutils.Unset, None, typing.Pattern[str]]:
+            value: typing.Union[str, typing.Pattern[str], usertypes.Unset]
+    ) -> typing.Union[usertypes.Unset, None, typing.Pattern[str]]:
         """Get a compiled regex from either a string or a regex object."""
         self._basic_py_validation(value, (str, self._regex_type))
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1421,10 +1443,10 @@ class Dict(BaseType):
 
     def to_py(
             self,
-            value: typing.Union[typing.Dict, configutils.Unset, None]
-    ) -> typing.Union[typing.Dict, configutils.Unset]:
+            value: typing.Union[typing.Dict, usertypes.Unset, None]
+    ) -> typing.Union[typing.Dict, usertypes.Unset]:
         self._basic_py_validation(value, dict)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return self._fill_fixed_keys({})
@@ -1473,7 +1495,7 @@ class File(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1505,7 +1527,7 @@ class Directory(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1526,16 +1548,23 @@ class Directory(BaseType):
 
 class FormatString(BaseType):
 
-    """A string with placeholders."""
+    """A string with placeholders.
+
+    Attributes:
+        fields: Which replacements are allowed in the format string.
+        completions: completions to be used, or None
+    """
 
     def __init__(self, fields: typing.Iterable[str],
-                 none_ok: bool = False) -> None:
+                 none_ok: bool = False,
+                 completions: _Completions = None) -> None:
         super().__init__(none_ok)
         self.fields = fields
+        self._completions = completions
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1550,6 +1579,12 @@ class FormatString(BaseType):
 
         return value
 
+    def complete(self) -> _Completions:
+        if self._completions is not None:
+            return self._completions
+        else:
+            return super().complete()
+
     def __repr__(self) -> str:
         return utils.get_repr(self, none_ok=self.none_ok, fields=self.fields)
 
@@ -1573,10 +1608,10 @@ class ShellCommand(List):
 
     def to_py(
             self,
-            value: typing.Union[typing.List, configutils.Unset],
-    ) -> typing.Union[typing.List, configutils.Unset]:
+            value: typing.Union[typing.List, usertypes.Unset],
+    ) -> typing.Union[typing.List, usertypes.Unset]:
         py_value = super().to_py(value)
-        if isinstance(py_value, configutils.Unset):
+        if isinstance(py_value, usertypes.Unset):
             return py_value
         elif not py_value:
             return []
@@ -1607,9 +1642,9 @@ class Proxy(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, QNetworkProxy, _SystemProxy]:
+    ) -> typing.Union[usertypes.Unset, None, QNetworkProxy, _SystemProxy]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1649,7 +1684,7 @@ class SearchEngineUrl(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1679,7 +1714,7 @@ class FuzzyUrl(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1715,10 +1750,10 @@ class Padding(Dict):
 
     def to_py(  # type: ignore
             self,
-            value: typing.Union[configutils.Unset, typing.Dict, None],
-    ) -> typing.Union[configutils.Unset, PaddingValues]:
+            value: typing.Union[usertypes.Unset, typing.Dict, None],
+    ) -> typing.Union[usertypes.Unset, PaddingValues]:
         d = super().to_py(value)
-        if isinstance(d, configutils.Unset):
+        if isinstance(d, usertypes.Unset):
             return d
 
         return PaddingValues(**d)
@@ -1730,7 +1765,7 @@ class Encoding(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1790,9 +1825,9 @@ class Url(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, QUrl]:
+    ) -> typing.Union[usertypes.Unset, None, QUrl]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1810,7 +1845,7 @@ class SessionName(BaseType):
 
     def to_py(self, value: _StrUnset) -> _StrUnsetNone:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1860,10 +1895,10 @@ class ConfirmQuit(FlagList):
 
     def to_py(
             self,
-            value: typing.Union[configutils.Unset, typing.List],
-    ) -> typing.Union[typing.List, configutils.Unset]:
+            value: typing.Union[usertypes.Unset, typing.List],
+    ) -> typing.Union[typing.List, usertypes.Unset]:
         values = super().to_py(value)
-        if isinstance(values, configutils.Unset):
+        if isinstance(values, usertypes.Unset):
             return values
         elif not values:
             return []
@@ -1904,9 +1939,9 @@ class Key(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, keyutils.KeySequence]:
+    ) -> typing.Union[usertypes.Unset, None, keyutils.KeySequence]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
@@ -1928,9 +1963,9 @@ class UrlPattern(BaseType):
     def to_py(
             self,
             value: _StrUnset
-    ) -> typing.Union[configutils.Unset, None, urlmatch.UrlPattern]:
+    ) -> typing.Union[usertypes.Unset, None, urlmatch.UrlPattern]:
         self._basic_py_validation(value, str)
-        if isinstance(value, configutils.Unset):
+        if isinstance(value, usertypes.Unset):
             return value
         elif not value:
             return None
