@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -27,6 +27,7 @@ import pdb  # noqa: T002
 import signal
 import functools
 import faulthandler
+import typing
 try:
     # WORKAROUND for segfaults when using pdb in pytest for some reason...
     import readline  # pylint: disable=unused-import
@@ -162,6 +163,10 @@ class CrashHandler(QObject):
                                                       all_objects)
         self._crash_dialog.show()
 
+    @pyqtSlot()
+    def shutdown(self):
+        self.destroy_crashlogfile()
+
     def destroy_crashlogfile(self):
         """Clean up the crash log file and delete it."""
         if self._crash_log_file is None:
@@ -232,10 +237,11 @@ class CrashHandler(QObject):
         self._quitter.quit_status['crash'] = False
         info = self._get_exception_info()
 
-        try:
-            ipc.server.ignored = True
-        except Exception:
-            log.destroy.exception("Error while ignoring ipc")
+        if ipc.server is not None:
+            try:
+                ipc.server.ignored = True
+            except Exception:
+                log.destroy.exception("Error while ignoring ipc")
 
         try:
             self._app.lastWindowClosed.disconnect(
@@ -291,9 +297,10 @@ class SignalHandler(QObject):
         self._quitter = quitter
         self._notifier = None
         self._timer = usertypes.Timer(self, 'python_hacks')
-        self._orig_handlers = {}
+        self._orig_handlers = {
+        }  # type: typing.MutableMapping[int, signal._HANDLER]
         self._activated = False
-        self._orig_wakeup_fd = None
+        self._orig_wakeup_fd = None  # type: typing.Optional[int]
 
     def activate(self):
         """Set up signal handlers.
@@ -318,7 +325,8 @@ class SignalHandler(QObject):
                 fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
             self._notifier = QSocketNotifier(read_fd, QSocketNotifier.Read,
                                              self)
-            self._notifier.activated.connect(self.handle_signal_wakeup)
+            self._notifier.activated.connect(  # type: ignore
+                self.handle_signal_wakeup)
             self._orig_wakeup_fd = signal.set_wakeup_fd(write_fd)
             # pylint: enable=import-error,no-member,useless-suppression
         else:
@@ -331,6 +339,7 @@ class SignalHandler(QObject):
         if not self._activated:
             return
         if self._notifier is not None:
+            assert self._orig_wakeup_fd is not None
             self._notifier.setEnabled(False)
             rfd = self._notifier.socket()
             wfd = signal.set_wakeup_fd(self._orig_wakeup_fd)
@@ -349,6 +358,7 @@ class SignalHandler(QObject):
 
         Python will get control here, so the signal will get handled.
         """
+        assert self._notifier is not None
         log.destroy.debug("Handling signal wakeup!")
         self._notifier.setEnabled(False)
         read_fd = self._notifier.socket()
