@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -30,6 +30,7 @@ import collections
 import enum
 import datetime
 import getpass
+import typing
 
 import attr
 import pkg_resources
@@ -44,11 +45,6 @@ try:
 except ImportError:  # pragma: no cover
     qWebKitVersion = None  # type: ignore  # noqa: N816
 
-try:
-    from PyQt5.QtWebEngineWidgets import QWebEngineProfile
-except ImportError:  # pragma: no cover
-    QWebEngineProfile = None  # type: ignore
-
 import qutebrowser
 from qutebrowser.utils import log, utils, standarddir, usertypes, message
 from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin
@@ -66,20 +62,20 @@ class DistributionInfo:
 
     """Information about the running distribution."""
 
-    id = attr.ib()
-    parsed = attr.ib()
-    version = attr.ib()
-    pretty = attr.ib()
+    id = attr.ib()  # type: typing.Optional[str]
+    parsed = attr.ib()  # type: Distribution
+    version = attr.ib()  # type: typing.Optional[typing.Tuple[str, ...]]
+    pretty = attr.ib()  # type: str
 
 
 pastebin_url = None
 Distribution = enum.Enum(
     'Distribution', ['unknown', 'ubuntu', 'debian', 'void', 'arch',
                      'gentoo', 'fedora', 'opensuse', 'linuxmint', 'manjaro',
-                     'kde'])
+                     'kde_flatpak'])
 
 
-def distribution():
+def distribution() -> typing.Optional[DistributionInfo]:
     """Get some information about the running Linux distribution.
 
     Returns:
@@ -104,27 +100,41 @@ def distribution():
     pretty = info.get('PRETTY_NAME', None)
     if pretty in ['Linux', None]:  # Funtoo has PRETTY_NAME=Linux
         pretty = info.get('NAME', 'Unknown')
+    assert pretty is not None
 
     if 'VERSION_ID' in info:
-        dist_version = pkg_resources.parse_version(info['VERSION_ID'])
+        dist_version = pkg_resources.parse_version(
+            info['VERSION_ID']
+        )  # type: typing.Optional[typing.Tuple[str, ...]]
     else:
         dist_version = None
 
     dist_id = info.get('ID', None)
     id_mappings = {
         'funtoo': 'gentoo',  # does not have ID_LIKE=gentoo
-        'org.kde.Platform': 'kde',
+        'org.kde.Platform': 'kde_flatpak',
     }
-    try:
-        parsed = Distribution[id_mappings.get(dist_id, dist_id)]
-    except KeyError:
-        parsed = Distribution.unknown
+
+    parsed = Distribution.unknown
+    if dist_id is not None:
+        try:
+            parsed = Distribution[id_mappings.get(dist_id, dist_id)]
+        except KeyError:
+            pass
 
     return DistributionInfo(parsed=parsed, version=dist_version, pretty=pretty,
                             id=dist_id)
 
 
-def _git_str():
+def is_sandboxed() -> bool:
+    """Whether the environment has restricted access to the host system."""
+    current_distro = distribution()
+    if current_distro is None:
+        return False
+    return current_distro.parsed == Distribution.kde_flatpak
+
+
+def _git_str() -> typing.Optional[str]:
     """Try to find out git version.
 
     Return:
@@ -150,7 +160,7 @@ def _git_str():
         return None
 
 
-def _git_str_subprocess(gitpath):
+def _git_str_subprocess(gitpath: str) -> typing.Optional[str]:
     """Try to get the git commit ID and timestamp by calling git.
 
     Args:
@@ -176,7 +186,7 @@ def _git_str_subprocess(gitpath):
         return None
 
 
-def _release_info():
+def _release_info() -> typing.Sequence[typing.Tuple[str, str]]:
     """Try to gather distribution release information.
 
     Return:
@@ -200,7 +210,7 @@ def _release_info():
     return data
 
 
-def _module_versions():
+def _module_versions() -> typing.Sequence[str]:
     """Get versions of optional modules.
 
     Return:
@@ -219,7 +229,7 @@ def _module_versions():
         ('PyQt5.QtWebEngineWidgets', []),
         ('PyQt5.QtWebEngine', ['PYQT_WEBENGINE_VERSION_STR']),
         ('PyQt5.QtWebKitWidgets', []),
-    ])
+    ])  # type: typing.Mapping[str, typing.Sequence[str]]
     for modname, attributes in modules.items():
         try:
             module = importlib.import_module(modname)
@@ -239,7 +249,7 @@ def _module_versions():
     return lines
 
 
-def _path_info():
+def _path_info() -> typing.Mapping[str, str]:
     """Get info about important path names.
 
     Return:
@@ -258,7 +268,7 @@ def _path_info():
     return info
 
 
-def _os_info():
+def _os_info() -> typing.Sequence[str]:
     """Get operating system info.
 
     Return:
@@ -272,11 +282,11 @@ def _os_info():
     elif utils.is_windows:
         osver = ', '.join(platform.win32_ver())
     elif utils.is_mac:
-        release, versioninfo, machine = platform.mac_ver()
-        if all(not e for e in versioninfo):
+        release, info_tpl, machine = platform.mac_ver()
+        if all(not e for e in info_tpl):
             versioninfo = ''
         else:
-            versioninfo = '.'.join(versioninfo)
+            versioninfo = '.'.join(info_tpl)
         osver = ', '.join([e for e in [release, versioninfo, machine] if e])
     elif utils.is_posix:
         osver = ' '.join(platform.uname())
@@ -289,7 +299,7 @@ def _os_info():
     return lines
 
 
-def _pdfjs_version():
+def _pdfjs_version() -> str:
     """Get the pdf.js version.
 
     Return:
@@ -315,7 +325,7 @@ def _pdfjs_version():
         return '{} ({})'.format(pdfjs_version, file_path)
 
 
-def _chromium_version():
+def _chromium_version() -> str:
     """Get the Chromium version for QtWebEngine.
 
     This can also be checked by looking at this file with the right Qt tag:
@@ -345,30 +355,31 @@ def _chromium_version():
 
     Qt 5.12: Chromium 69
     (LTS)    69.0.3497.113 (2018-09-27)
-             5.12.5: Security fixes up to 76.0.3809.87 (2019-06-30)
+             5.12.6: Security fixes up to 76.0.3809.87 (2019-07-30)
+                     plus fix for CVE-2019-13720 from Chrome 78
 
     Qt 5.13: Chromium 73
              73.0.3683.105 (~2019-02-28)
-             5.13.1: Security fixes up to 76.0.3809.87 (2019-06-30)
+             5.13.2: Security fixes up to 77.0.3865.120 (2019-10-10)
+
+    Qt 5.14: Chromium 77
+             77.0.3865.129 (~2019-10-10)
+             5.14.0: Security fixes up to 78.0.3904.108 (2019-11-18)
 
     Also see https://www.chromium.org/developers/calendar
     and https://chromereleases.googleblog.com/
     """
-    if webenginesettings is None or QWebEngineProfile is None:  # type: ignore
-        # This should never happen
+    if webenginesettings is None:
         return 'unavailable'  # type: ignore
-    ua = webenginesettings.default_user_agent
-    if ua is None:
-        profile = QWebEngineProfile.defaultProfile()
-        ua = profile.httpUserAgent()
-    match = re.search(r' Chrome/([^ ]*) ', ua)
-    if not match:
-        log.misc.error("Could not get Chromium version from: {}".format(ua))
-        return 'unknown'
-    return match.group(1)
+
+    if webenginesettings.parsed_user_agent is None:
+        webenginesettings.init_user_agent()
+        assert webenginesettings.parsed_user_agent is not None
+
+    return webenginesettings.parsed_user_agent.upstream_browser_version
 
 
-def _backend():
+def _backend() -> str:
     """Get the backend line with relevant information."""
     if objects.backend == usertypes.Backend.QtWebKit:
         return 'new QtWebKit (WebKit {})'.format(qWebKitVersion())
@@ -397,20 +408,20 @@ def _config_py_loaded() -> str:
         return "no config.py was loaded"
 
 
-def version():
+def version() -> str:
     """Return a string with various version information."""
     lines = ["qutebrowser v{}".format(qutebrowser.__version__)]
     gitver = _git_str()
     if gitver is not None:
         lines.append("Git commit: {}".format(gitver))
 
-    lines.append("Backend: {}".format(_backend()))
+    lines.append('Backend: {}'.format(_backend()))
+    lines.append('Qt: {}'.format(earlyinit.qt_version()))
 
     lines += [
         '',
         '{}: {}'.format(platform.python_implementation(),
                         platform.python_version()),
-        'Qt: {}'.format(earlyinit.qt_version()),
         'PyQt: {}'.format(PYQT_VERSION_STR),
         '',
     ]
@@ -471,7 +482,7 @@ def version():
     return '\n'.join(lines)
 
 
-def opengl_vendor():  # pragma: no cover
+def opengl_vendor() -> typing.Optional[str]:  # pragma: no cover
     """Get the OpenGL vendor used.
 
     This returns a string such as 'nouveau' or
@@ -485,7 +496,8 @@ def opengl_vendor():  # pragma: no cover
         log.init.debug("Using override {}".format(override))
         return override
 
-    old_context = QOpenGLContext.currentContext()
+    old_context = typing.cast(typing.Optional[QOpenGLContext],
+                              QOpenGLContext.currentContext())
     old_surface = None if old_context is None else old_context.surface()
 
     surface = QOffscreenSurface()
@@ -527,20 +539,22 @@ def opengl_vendor():  # pragma: no cover
             old_context.makeCurrent(old_surface)
 
 
-def pastebin_version(pbclient=None):
+def pastebin_version(pbclient: pastebin.PastebinClient = None) -> None:
     """Pastebin the version and log the url to messages."""
-    def _yank_url(url):
+    def _yank_url(url: str) -> None:
         utils.set_clipboard(url)
         message.info("Version url {} yanked to clipboard.".format(url))
 
-    def _on_paste_version_success(url):
+    def _on_paste_version_success(url: str) -> None:
+        assert pbclient is not None
         global pastebin_url
         url = url.strip()
         _yank_url(url)
         pbclient.deleteLater()
         pastebin_url = url
 
-    def _on_paste_version_err(text):
+    def _on_paste_version_err(text: str) -> None:
+        assert pbclient is not None
         message.error("Failed to pastebin version"
                       " info: {}".format(text))
         pbclient.deleteLater()
