@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 
 # This file is part of qutebrowser.
 #
@@ -52,14 +52,6 @@ fg_colors = {
 bg_colors = {name: col + 10 for name, col in fg_colors.items()}
 
 
-term_attributes = {
-    'bright': 1,
-    'dim': 2,
-    'normal': 22,
-    'reset': 0,
-}
-
-
 def _esc(code):
     """Get an ANSI color code based on a color number."""
     return '\033[{}m'.format(code)
@@ -86,16 +78,6 @@ def print_subtitle(text):
     print_col("------ {} ------".format(text), 'cyan')
 
 
-def print_bold(text):
-    """Print a bold text."""
-    if use_color:
-        bold = _esc(term_attributes['bright'])
-        reset = _esc(term_attributes['reset'])
-        print(''.join([bold, text, reset]))
-    else:
-        print(text)
-
-
 def change_cwd():
     """Change the scripts cwd if it was started inside the script folder."""
     cwd = os.getcwd()
